import hashlib
import os
import pathlib
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
# sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))
import dpdispatcher
from dpdispatcher.base_context import BaseContext
from dpdispatcher.distributed_shell import DistributedShell
from dpdispatcher.dp_cloud_server import Lebesgue
from dpdispatcher.hdfs_cli import HDFS
from dpdispatcher.hdfs_context import HDFSContext

# from dpgen.dispatcher.Dispatcher import FinRecord
from dpdispatcher.JobStatus import JobStatus

# from dpdispatcher.local_context import local_context
from dpdispatcher.lazy_local_context import LazyLocalContext

# from dpdispatcher.local_context import LocalSession
from dpdispatcher.local_context import LocalContext, _identical_files
from dpdispatcher.lsf import LSF
from dpdispatcher.pbs import PBS
from dpdispatcher.shell import Shell
from dpdispatcher.slurm import Slurm
from dpdispatcher.ssh_context import SSHContext, SSHSession

try:
    from dpdispatcher.dp_cloud_server import DpCloudServer
    from dpdispatcher.dp_cloud_server_context import DpCloudServerContext
except Exception:
    pass
import dargs

from dpdispatcher.machine import Machine
from dpdispatcher.submission import Job, Resources, Submission, Task
from dpdispatcher.utils import RetrySignal, retry


def setUpModule():
    os.chdir(os.path.abspath(os.path.dirname(__file__)))


def get_file_md5(file_path):
    return hashlib.md5(pathlib.Path(file_path).read_bytes()).hexdigest()



import json
import os
import shutil
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
import unittest

from .context import (
    Job,
    LocalContext,
    Machine,
    Resources,
    Shell,
    Submission,
    Task,
    get_file_md5,
)


@unittest.skipIf(sys.platform == "win32", "Shell is not supported on Windows")
class TestShellCudaMultiDevices(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None

    def test_shell_cuda_multi_devices(self):
        with open("jsons/machine_if_cuda_multi_devices.json", "r") as f:
            machine_dict = json.load(f)
        machine = Machine.load_from_dict(machine_dict["machine"])
        resources = Resources.load_from_dict(machine_dict["resources"])

        task_list = []
        for ii in range(16):
            task = Task(
                command=f"echo dpdispatcher_unittest_{ii}",
                task_work_path="./",
                forward_files=[],
                backward_files=[],
                outlog="out.txt",
            )
            task_list.append(task)

        submission = Submission(
            work_base="test_dir/",
            machine=machine,
            resources=resources,
            forward_common_files=["test.txt"],
            backward_common_files=["out.txt"],
            task_list=task_list,
        )
        submission.run_submission(clean=False)

        for ii in ["test.txt"]:
            f1 = os.path.join("test_if_cuda_multi_devices/", "test_dir/", ii)
            f2 = os.path.join(
                "tmp_if_cuda_multi_devices/", submission.submission_hash, ii
            )
            self.assertEqual(get_file_md5(f1), get_file_md5(f2))

        self.assertTrue(os.path.isfile("test_if_cuda_multi_devices/test_dir/out.txt"))

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree("tmp_if_cuda_multi_devices/")
        # pass



#!/usr/bin/env python3
import glob
import json
import os
import shutil
import sys
import time
import unittest
import uuid

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
# from .context import LocalContext
from dpdispatcher.local_context import LocalContext

from .context import Job, JobStatus, Resources, Submission, Task
from .sample_class import SampleClass

task_dict = SampleClass.get_sample_task_dict()
assert os.path.isfile("jsons/task.json") is False
with open("jsons/task.json", "w") as f:
    json.dump(task_dict, f, indent=4)

resources_dict = SampleClass.get_sample_resources_dict()
assert os.path.isfile("jsons/resources.json") is False
with open("jsons/resources.json", "w") as f:
    json.dump(resources_dict, f, indent=4)

pbs = SampleClass.get_sample_pbs_local_context()
submission = SampleClass.get_sample_submission()
submission.bind_machine(machine=pbs)
assert os.path.isfile("jsons/submission.json") is False
with open("jsons/submission.json", "w") as f:
    json.dump(submission.serialize(), f, indent=4)

job_dict = SampleClass.get_sample_job_dict()
assert os.path.isfile("jsons/job.json") is False
with open("jsons/job.json", "w") as f:
    json.dump(job_dict, f, indent=4)



import os
import socket
import sys
import unittest

from paramiko.ssh_exception import SSHException

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
from .context import Machine, Resources, SSHSession, Submission, Task
from .sample_class import SampleClass


@unittest.skipIf(
    os.environ.get("DPDISPATCHER_TEST") != "ssh", "outside the ssh testing environment"
)
class TestSSHContext(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        mdata = {
            "batch_type": "Shell",
            "context_type": "SSHContext",
            "local_root": "./test_context_dir",
            "remote_root": "/dpdispatcher_working",
            "remote_profile": {
                "hostname": "server",
                "port": 22,
                "username": "root",
                "key_filename": "/root/.ssh/id_rsa",
            },
        }
        try:
            cls.machine = Machine.load_from_dict(mdata)
        except (SSHException, socket.timeout):
            raise unittest.SkipTest("SSHException ssh cannot connect")
        cls.submission = SampleClass.get_sample_submission()
        cls.submission.bind_machine(cls.machine)
        cls.submission_hash = cls.submission.submission_hash
        file_list = [
            "bct-1/log.lammps",
            "bct-2/log.lammps",
            "bct-3/log.lammps",
            "bct-4/log.lammps",
            "dir with space/file with space",
        ]
        for file in file_list:
            cls.machine.context.sftp.mkdir(
                os.path.join(cls.machine.context.remote_root, os.path.dirname(file))
            )
            cls.machine.context.write_file(file, "# mock log")

    @classmethod
    def tearDownClass(cls):
        cls.machine.context.clean()
        # close the server
        cls.machine.context.close()

    def setUp(self):
        self.context = self.__class__.machine.context

    def test_ssh_session(self):
        self.assertIsInstance(self.__class__.machine.context.ssh_session, SSHSession)

    def test_upload(self):
        self.context.upload(self.__class__.submission)
        check_file_list = [
            "graph.pb",
            "bct-1/conf.lmp",
            "bct-4/input.lammps",
            "dir with space/file with space",
        ]
        for file in check_file_list:
            self.assertTrue(
                self.context.check_file_exists(
                    os.path.join(self.context.remote_root, file)
                )
            )

    def test_empty_transfer(self):
        # Both forward_files and backward_files are empty
        machine = Machine.load_from_dict(self.machine.serialize())
        resources = Resources.load_from_dict(
            {
                "number_node": 1,
                "cpu_per_node": 1,
                "gpu_per_node": 0,
                "queue_name": "?",
                "group_size": 2,
            }
        )
        task = Task(
            command=f"echo dpdispatcher_unittest",
            task_work_path="./",
            forward_files=[],
            backward_files=[],
            outlog=f"out.txt",
        )

        submission = Submission(
            work_base="./",
            machine=machine,
            resources=resources,
            forward_common_files=[],
            backward_common_files=[],
            task_list=[task],
        )
        submission.run_submission()

    def test_download(self):
        self.context.download(self.__class__.submission)


@unittest.skipIf(
    os.environ.get("DPDISPATCHER_TEST") != "ssh", "outside the ssh testing environment"
)
class TestSSHContextNoCompress(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        mdata = {
            "batch_type": "Shell",
            "context_type": "SSHContext",
            "local_root": "./test_context_dir",
            "remote_root": "/dpdispatcher_working",
            "remote_profile": {
                "hostname": "server",
                "port": 22,
                "username": "root",
                "tar_compress": False,
            },
        }
        try:
            cls.machine = Machine.load_from_dict(mdata)
        except (SSHException, socket.timeout):
            raise unittest.SkipTest("SSHException ssh cannot connect")
        cls.submission = SampleClass.get_sample_submission()
        cls.submission.bind_machine(cls.machine)
        cls.submission_hash = cls.submission.submission_hash
        file_list = [
            "bct-1/log.lammps",
            "bct-2/log.lammps",
            "bct-3/log.lammps",
            "bct-4/log.lammps",
            "dir with space/file with space",
        ]
        for file in file_list:
            cls.machine.context.sftp.mkdir(
                os.path.join(cls.machine.context.remote_root, os.path.dirname(file))
            )
            cls.machine.context.write_file(file, "# mock log")

    @classmethod
    def tearDownClass(cls):
        cls.machine.context.clean()
        # close the server
        cls.machine.context.close()

    def setUp(self):
        self.context = self.__class__.machine.context

    def test_ssh_session(self):
        self.assertIsInstance(self.__class__.machine.context.ssh_session, SSHSession)

    def test_upload(self):
        self.context.upload(self.__class__.submission)
        check_file_list = [
            "graph.pb",
            "bct-1/conf.lmp",
            "bct-4/input.lammps",
            "dir with space/file with space",
        ]
        for file in check_file_list:
            self.assertTrue(
                self.context.check_file_exists(
                    os.path.join(self.context.remote_root, file)
                )
            )

    def test_download(self):
        self.context.download(self.__class__.submission)



import glob
import os
from zipfile import ZipFile

# def zip_file_list(root_path, zip_filename, file_list=[]):
#     shutil.make_archive(base_name=zip_filename,
#         root_dir=root_path,)


def zip_file_list(root_path, zip_filename, file_list=[]):
    out_zip_file = os.path.join(root_path, zip_filename)
    # print('debug: file_list', file_list)
    zip_obj = ZipFile(out_zip_file, "w")
    for f in file_list:
        matched_files = os.path.join(root_path, f)
        for ii in glob.glob(matched_files):
            # print('debug: matched_files:ii', ii)
            if os.path.isdir(ii):
                arcname = os.path.relpath(ii, start=root_path)
                zip_obj.write(ii, arcname)
                for root, dirs, files in os.walk(ii):
                    for file in files:
                        filename = os.path.join(root, file)
                        arcname = os.path.relpath(filename, start=root_path)
                        # print('debug: filename:arcname:root_path', filename, arcname, root_path)
                        zip_obj.write(filename, arcname)
            else:
                arcname = os.path.relpath(ii, start=root_path)
                zip_obj.write(ii, arcname)
    zip_obj.close()
    return out_zip_file


# def zip_files(root_path, out_file, selected=[]):
#     obj = ZipFile(out_file, "w")
#     # change /xxx/ to /xxx or xxx to /xxx and pop ''
#     for i in range(len(selected)):
#         if not selected[i]:
#             selected.pop(i)
#             continue

#         selected[i] = selected[i].strip()
#         if selected[i].endswith('/'):
#             selected[i] = selected[i][:-1]
#         if not selected[i].startswith('/'):
#             selected[i] = '/{}'.format(selected[i])

#     for root, dirs, files in os.walk(root_path):
#         for item in files:
#             filename = os.path.join(root, item)
#             arcname = filename.replace(root_path,'')
#             if not is_selected(arcname, selected):
#                 continue

#             obj.write(filename, arcname)
#     if not obj.filelist:
#         return

#     obj.close()


# def is_selected(arcname, selected):
#     if not selected:
#         return True

#     arcdir = os.path.dirname(arcname)
#     for s in selected:
#         if arcname == s:
#             return True

#         if arcdir == s:
#             return True

#         if arcname.startswith(s + '/'):
#             return True

#     return False


def unzip_file(zip_file, out_dir="./"):
    obj = ZipFile(zip_file, "r")
    for item in obj.namelist():
        obj.extract(item, out_dir)



import glob
import json
import os
import shutil
import sys
import time
import unittest
import uuid
from unittest.mock import MagicMock, PropertyMock, patch

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
# from .context import LocalSession
# from .context import LocalContext
# from dpdispatcher.local_context import LocalContext
from .context import (
    PBS,
    Job,
    JobStatus,
    LocalContext,
    Resources,
    Slurm,
    Submission,
    Task,
)


class SampleClass(object):
    @classmethod
    def get_sample_resources(cls):
        resources = Resources(
            number_node=1,
            cpu_per_node=4,
            gpu_per_node=1,
            queue_name="T4_4_15",
            group_size=2,
            custom_flags=[],
            strategy={"if_cuda_multi_devices": False},
            para_deg=1,
            module_unload_list=[],
            module_list=[],
            source_list=[],
            envs={},
        )
        return resources

    @classmethod
    def get_sample_resources_dict(cls):
        resources_dict = {
            "number_node": 1,
            "cpu_per_node": 4,
            "gpu_per_node": 1,
            "queue_name": "T4_4_15",
            "group_size": 2,
            "custom_flags": [],
            "strategy": {"if_cuda_multi_devices": False, "ratio_unfinished": 0.0},
            "para_deg": 1,
            "module_purge": False,
            "module_unload_list": [],
            "module_list": [],
            "source_list": [],
            "envs": {},
            "prepend_script": [],
            "append_script": [],
            "wait_time": 0,
            "kwargs": {},
        }
        return resources_dict

    @classmethod
    def get_sample_task(cls):
        task = Task(
            command="lmp -i input.lammps",
            task_work_path="bct-1/",
            forward_files=["conf.lmp", "input.lammps"],
            backward_files=["log.lammps"],
            outlog="log",
            errlog="err",
        )
        return task

    @classmethod
    def get_sample_task_dict(cls):
        task_dict = {
            "command": "lmp -i input.lammps",
            "task_work_path": "bct-1/",
            "forward_files": ["conf.lmp", "input.lammps"],
            "backward_files": ["log.lammps"],
            "outlog": "log",
            "errlog": "err",
        }
        return task_dict

    @classmethod
    def get_sample_task_list(cls):
        task1 = Task(
            command="lmp -i input.lammps",
            task_work_path="bct-1/",
            forward_files=["conf.lmp", "input.lammps"],
            backward_files=["log.lammps"],
        )
        task2 = Task(
            command="lmp -i input.lammps",
            task_work_path="bct-2/",
            forward_files=["conf.lmp", "input.lammps"],
            backward_files=["log.lammps"],
        )
        task3 = Task(
            command="lmp -i input.lammps",
            task_work_path="bct-3/",
            forward_files=["conf.lmp", "input.lammps"],
            backward_files=["log.lammps"],
        )
        task4 = Task(
            command="lmp -i input.lammps",
            task_work_path="bct-4/",
            forward_files=["conf.lmp", "input.lammps"],
            backward_files=["log.lammps"],
        )
        task_list = [task1, task2, task3, task4]
        return task_list

    @classmethod
    def get_sample_empty_submission(cls):
        resources = cls.get_sample_resources()
        # print(task_list)
        empty_submission = Submission(
            work_base="0_md/",
            machine=None,
            resources=resources,
            forward_common_files=["graph.pb"],
            backward_common_files=[],
        )
        # print('SampleClass.get_sample_empty_submission:', empty_submission)
        # print('SampleClass.get_sample_empty_submission.belonging_tasks:', empty_submission.belonging_tasks)
        return empty_submission

    @classmethod
    def get_sample_submission(cls):
        submission = cls.get_sample_empty_submission()
        task_list = cls.get_sample_task_list()
        submission.register_task_list(task_list)
        submission.generate_jobs()
        return submission

    @classmethod
    def get_sample_submission_dict(cls):
        submission = cls.get_sample_submission()
        submission_dict = submission.serialize()
        return submission_dict

    @classmethod
    def get_sample_job(cls):
        Submission = cls.get_sample_submission()
        job = Submission.belonging_jobs[0]
        return job

    @classmethod
    def get_sample_job_dict(cls):
        job = cls.get_sample_job()
        job_dict = job.serialize()
        return job_dict

    @classmethod
    def get_sample_pbs_local_context(cls):
        # local_session = LocalSession({'work_path':'test_work_path/'})
        local_context = LocalContext(
            local_root="test_pbs_dir/", remote_root="tmp_pbs_dir/"
        )
        pbs = PBS(context=local_context)
        return pbs

    @classmethod
    def get_sample_slurm_local_context(cls):
        # local_session = LocalSession({'work_path':'test_work_path/'})
        local_context = LocalContext(
            local_root="test_slurm_dir/", remote_root="tmp_slurm_dir/"
        )
        slurm = Slurm(context=local_context)
        return slurm



import json
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
# from .sample_class import SampleClass
# from .context import Machine
# from .context import Resources
# from dpdispatcher.local_context import LocalSession
# from dpdispatcher.local_context import LocalContext
# from dpdispatcher.lazy_local_context import LazyLocalContext

from dpdispatcher.machine import Machine
from dpdispatcher.submission import Job, Resources, Submission, Task
from tests.sample_class import SampleClass

# from dpdispatcher.pbs import PBS

# local_session = LocalSession({'work_path':'test_work_path/'})
# local_context = LocalContext(local_root='test_pbs_dir/', work_profile=local_session)

# lazy_local_context = LazyLocalContext(local_root='/home/fengbo/10_dpdispatcher/dpdispatcher/tests/test_pbs_dir', work_profile=None)

# ssh_session = SSHSession(hostname='39.106.84.25', remote_root='/home/fengbo/dp_remote', username='fengbo')
# ssh_context = SSHContext(local_root='test_slurm_dir', ssh_session=ssh_session)

# pbs = PBS(context=ssh_context)
# pbs = PBS(context=local_context)
# pbs = PBS(context=lazy_local_context)

submission = SampleClass.get_sample_submission()
# pbs = SampleClass.get_sample_pbs_local_context()
# slurm = SampleClass.get_sample_slurm_local_context()

with open("jsons/machine_ali_ehpc.json", "r") as f:
    compute_dict = json.load(f)

machine = Machine.load_from_dict(compute_dict["machine"])
resources = Resources.load_from_dict(compute_dict["resources"])

submission.resouces = resources
submission.bind_machine(machine=machine)
# submission.run_submission()
# submission.run_submission(exit_on_submit=True)
submission.run_submission()


# resources = Resources(number_node=1, cpu_per_node=4, gpu_per_node=1, queue_name="V100_8_32", group_size=2, if_cuda_multi_devices=True)
# submission = Submission(work_base='0_md/', resources=resources,  forward_common_files=['graph.pb'], backward_common_files=[]) #,  batch=PBS)
# task1 = Task(command='lmp_serial -i input.lammps', task_work_path='bct-1/', forward_files=['conf.lmp', 'input.lammps'], backward_files=['log.lammps'], task_need_resources=1)
# task2 = Task(command='lmp_serial -i input.lammps', task_work_path='bct-2/', forward_files=['conf.lmp', 'input.lammps'], backward_files=['log.lammps'], task_need_resources=0.25)
# task3 = Task(command='lmp_serial -i input.lammps', task_work_path='bct-3/', forward_files=['conf.lmp', 'input.lammps'], backward_files=['log.lammps'], task_need_resources=0.25)
# task4 = Task(command='lmp_serial -i input.lammps', task_work_path='bct-4/', forward_files=['conf.lmp', 'input.lammps'], backward_files=['log.lammps'], task_need_resources=0.5)
# submission.register_task_list([task1, task2, task3, task4, ])
# submission.generate_jobs()
# submission.bind_batch(batch=pbs)
# for job in submission.belonging_jobs:
#     job.job_to_json()
# print('111', submission)
# submission2 = Submission.recover_jobs_from_json('./jr.json')
# print('222', submission2)
# print(submission==submission2)

# submission1.dump_jobs_fo_json()
# submission2 = Submission.submission_from_json('jsons/submission.json')
# print(677, submission==submission2)
# print(submission1.belonging_jobs)
# print(local_context)



import glob
import json
import os
import shutil
import sys
import time
import unittest
import uuid
from unittest.mock import MagicMock, PropertyMock, patch

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
from .context import PBS, Job, JobStatus, LocalContext, Resources, Submission, Task
from .sample_class import SampleClass

# print('in', SampleClass.get_sample_empty_submission())


class TestSubmissionInit(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        # self.empty_submission = SampleClass.get_sample_empty_submission()
        # print('TestSubmissionInit.setUp:self.empty_submission.belonging_tasks', self.empty_submission.belonging_tasks)

    def test_reigister_task(self):
        empty_submission = SampleClass.get_sample_empty_submission()
        task = SampleClass.get_sample_task()
        # print('TestSubmissionInit.test_reigister_task:self.empty_submission.belonging_tasks', empty_submission.belonging_tasks)
        empty_submission.register_task(task=task)
        # print('7890809', SampleClass.get_sample_empty_submission().belonging_tasks)
        self.assertEqual([task], empty_submission.belonging_tasks)

    def test_reigister_task_whether_copy(self):
        empty_submission = SampleClass.get_sample_empty_submission()
        task = SampleClass.get_sample_task()
        empty_submission.register_task(task=task)
        empty_submission2 = SampleClass.get_sample_empty_submission()
        self.assertEqual(empty_submission2.belonging_tasks, [])

        # empty_submission =

    # def test_reigister_task_list(self):
    #     pass


# print('out', SampleClass.get_sample_empty_submission())
# print('TestSubmissionInit.test_register_task_list:task_list', task_list)
# empty_submission = SampleClass.get_sample_empty_submission()
# task_list = SampleClass.get_sample_task_list()
# empty_submission.register_task_list(task_list=task_list)
# self.empty_submission.register_task_list(task_list=task_list)
# self.assertEqual(task_list, empty_submission.belonging_tasks)

# def tesk_generate_jobs(self):
#     task_list = SampleClass.get_sample_task_list()
#     self.submission.register_task_list(task_list=task_list)
#     self.submission.generate_jobs()
#     task1, task2, task3, task4 = task_list
#     task_ll = [job.job_task_list for job in self.submission.belonging_jobs]
#     self.assertEqual([[task3, task2], [task4, task1]], task_ll)



import glob
import json
import os
import shutil
import sys
import time
import unittest
import uuid

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "slurm"
from .context import (
    Dispatcher,
    JobStatus,
    LocalContext,
    LocalSession,
    Slurm,
    my_file_cmp,
    setUpModule,
)


@unittest.skipIf(not shutil.which("sbatch"), "requires Slurm")
class TestDispatcher(unittest.TestCase):
    def setUp(self):
        os.makedirs("loc", exist_ok=True)
        os.makedirs("rmt", exist_ok=True)
        os.makedirs("loc/task0", exist_ok=True)
        os.makedirs("loc/task1", exist_ok=True)
        os.makedirs("loc/task2", exist_ok=True)
        for ii in ["loc/task0", "loc/task1", "loc/task2"]:
            with open(os.path.join(ii, "test0"), "w") as fp:
                fp.write("this is test0 from " + ii + "\n")
        work_profile = {}
        self.disp = Dispatcher(work_profile, "lazy-local", "slurm")

    def tearDown(self):
        shutil.rmtree("loc")
        shutil.rmtree("rmt")
        if os.path.exists("dpgen.log"):
            os.remove("dpgen.log")

    def test_sub_success(self):
        tasks = ["task0", "task1", "task2"]
        self.disp.run_jobs(
            None,
            "cp test0 test1",
            "loc",
            tasks,
            2,
            [],
            ["test0"],
            ["test1", "hereout.log", "hereerr.log"],
            outlog="hereout.log",
            errlog="hereerr.log",
        )
        for ii in tasks:
            my_file_cmp(
                self, os.path.join("loc", ii, "test0"), os.path.join("loc", ii, "test1")
            )
            self.assertTrue(os.path.isfile(os.path.join("loc", ii, "hereout.log")))
            self.assertTrue(os.path.isfile(os.path.join("loc", ii, "hereerr.log")))



import glob
import json
import os
import shutil
import sys
import time
import unittest
import uuid
from socket import gaierror
from unittest.mock import MagicMock, PropertyMock, patch

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
from dargs.dargs import ArgumentKeyError

from .context import (
    LSF,
    PBS,
    BaseContext,
    DistributedShell,
    HDFSContext,
    JobStatus,
    LazyLocalContext,
    LocalContext,
    Machine,
    Shell,
    Slurm,
    SSHContext,
    dargs,
)
from .sample_class import SampleClass


class TestMachineInit(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None

    def test_machine_serialize_deserialize(self):
        pbs = SampleClass.get_sample_pbs_local_context()
        self.assertEqual(pbs, Machine.deserialize(pbs.serialize()))

    def test_machine_load_from_dict(self):
        pbs = SampleClass.get_sample_pbs_local_context()
        self.assertEqual(pbs, PBS.load_from_dict(pbs.serialize()))



import getpass
import glob
import json
import os
import pathlib
import shutil
import sys
import tarfile
import unittest
import uuid

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
from glob import glob

from .context import HDFS, HDFSContext, Machine
from .sample_class import SampleClass


@unittest.skipIf(not shutil.which("hadoop"), "requires hadoop")
class TestHDFSContext(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        with open("jsons/machine_yarn.json", "r") as f:
            mdata = json.load(f)
        cls.machine = Machine.load_from_dict(mdata["machine"])
        cls.submission = SampleClass.get_sample_submission()
        cls.submission.bind_machine(cls.machine)
        cls.submission_hash = cls.submission.submission_hash

    def setUp(self):
        self.context = self.__class__.machine.context

    def test_0_hdfs_context(self):
        self.assertIsInstance(self.context, HDFSContext)

    def test_1_upload(self):
        self.context.upload(self.__class__.submission)

    def test_2_fake_run(self):
        rfile_tgz = (
            self.context.remote_root
            + "/"
            + self.context.submission.submission_hash
            + "_upload.tgz"
        )
        tmp_dir = "./tmp_fake_run"
        if os.path.exists(tmp_dir):
            shutil.rmtree(tmp_dir)
        os.mkdir(tmp_dir)
        self.assertTrue(HDFS.copy_to_local(rfile_tgz, tmp_dir))

        cwd = os.getcwd()
        os.chdir(tmp_dir)
        tgz_file_list = glob("*_upload.tgz")
        for tgz in tgz_file_list:
            with tarfile.open(tgz, "r:gz") as tar:
                tar.extractall()
            os.remove(tgz)

        file_list = [
            "bct-1/log.lammps",
            "bct-2/log.lammps",
            "bct-3/log.lammps",
            "bct-4/log.lammps",
        ]
        for fname in file_list:
            with open(fname, "w") as fp:
                fp.write("# mock log")

        file_list = glob("./*")
        download_tgz = self.context.submission.submission_hash + "_1_download.tar.gz"
        with tarfile.open(download_tgz, "w:gz", dereference=True) as tar:
            for ii in file_list:
                tar.add(ii)
        ret, _ = HDFS.copy_from_local(download_tgz, self.context.remote_root)
        self.assertTrue(ret)
        os.chdir(cwd)
        shutil.rmtree(tmp_dir)

    def test_3_download(self):
        self.context.download(self.__class__.submission)
        file_list = [
            "bct-1/log.lammps",
            "bct-2/log.lammps",
            "bct-3/log.lammps",
            "bct-4/log.lammps",
        ]
        for fname in file_list:
            self.assertTrue(
                os.path.isfile(os.path.join(self.context.local_root, fname))
            )
            os.remove(os.path.join(self.context.local_root, fname))



from abc import ABCMeta, abstractmethod
from typing import List, Optional, Tuple

from dargs import Argument

from dpdispatcher import dlog


class BaseContext(metaclass=ABCMeta):
    subclasses_dict = {}
    options = set()
    # alias: for subclasses_dict key
    # notes: this attribute can be inherited
    alias: Tuple[str, ...] = tuple()

    def __new__(cls, *args, **kwargs):
        if cls is BaseContext:
            subcls = cls.subclasses_dict[kwargs["context_type"]]
            instance = subcls.__new__(subcls, *args, **kwargs)
        else:
            instance = object.__new__(cls)
        return instance

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        alias = [cls.__name__, *cls.alias]
        for aa in alias:
            cls.subclasses_dict[aa] = cls
            cls.subclasses_dict[aa.lower()] = cls
            cls.subclasses_dict[aa.replace("Context", "")] = cls
            cls.subclasses_dict[aa.lower().replace("context", "")] = cls
        cls.options.add(cls.__name__)

    @classmethod
    def load_from_dict(cls, context_dict):
        context_type = context_dict["context_type"]
        # print("debug778:context_type", cls.subclasses_dict, context_type)
        try:
            context_class = cls.subclasses_dict[context_type]
        except KeyError as e:
            dlog.error(
                f"KeyError:context_type; context_type:{context_type}; cls.subclasses_dict:{cls.subclasses_dict}"
            )
            raise e
        context = context_class.load_from_dict(context_dict)
        return context

    def bind_submission(self, submission):
        self.submission = submission

    @abstractmethod
    def upload(self, submission):
        raise NotImplementedError("abstract method")

    @abstractmethod
    def download(
        self, submission, check_exists=False, mark_failure=True, back_error=False
    ):
        raise NotImplementedError("abstract method")

    @abstractmethod
    def clean(self):
        raise NotImplementedError("abstract method")

    @abstractmethod
    def write_file(self, fname, write_str):
        raise NotImplementedError("abstract method")

    @abstractmethod
    def read_file(self, fname):
        raise NotImplementedError("abstract method")

    def kill(self, proc):
        raise NotImplementedError("abstract method")

    def check_finish(self, proc):
        raise NotImplementedError("abstract method")

    @classmethod
    def machine_arginfo(cls) -> Argument:
        """Generate the machine arginfo.

        Returns
        -------
        Argument
            machine arginfo
        """
        alias = []
        for aa in cls.alias:
            alias.extend(
                (
                    aa,
                    aa.lower(),
                    aa.replace("Context", ""),
                    aa.lower().replace("context", ""),
                )
            )
        return Argument(
            cls.__name__,
            dict,
            sub_fields=cls.machine_subfields(),
            alias=[
                cls.__name__.lower(),
                cls.__name__.replace("Context", ""),
                cls.__name__.lower().replace("context", ""),
                *alias,
            ],
        )

    @classmethod
    def machine_subfields(cls) -> List[Argument]:
        """Generate the machine subfields.

        Returns
        -------
        list[Argument]
            machine subfields
        """
        doc_remote_profile = "The information used to maintain the connection with remote machine. This field is empty for this context."
        return [Argument("remote_profile", dict, optional=True, doc=doc_remote_profile)]



import json
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

# from dpdispatcher.local_context import LocalSession
# from dpdispatcher.local_context import LocalContext

# from dpdispatcher.batch import Batch
# from dpdispatcher.pbs import PBS
from dpdispatcher.batch_object import BatchObject
from dpdispatcher.submission import Job, Resources, Submission, Task

with open("ssh_machine.json", "r") as f:
    jdata = json.load(f)

batch = BatchObject(jdata=jdata)
# local_session = LocalSession({'work_path':'temp2'})
# local_context = LocalContext(local_root='temp1/', work_profile=local_session)
# lazy_local_context = LazyLocalContext(local_root='/home/fengbo/10_dpdispatcher/dpdispatcher/tests/temp3/0_md', work_profile=None)
# pbs = PBS(context=lazy_local_context)
# ssh_session = SSHSession(hostname='39.106.84.25', remote_root='/home/fengbo/dp_remote', username='fengbo')
# ssh_context = SSHContext(local_root='test_slurm_dir', ssh_session=ssh_session)
# jdata =

# pbs = PBS(context=ssh_context)

resources = Resources(
    number_node=1, cpu_per_node=4, gpu_per_node=1, queue_name="V100_8_32", group_size=4
)
submission = Submission(
    work_base="test_pbs_dir/0_md",
    resources=resources,
    forward_common_files=["graph.pb"],
    backward_common_files=[],
)  # ,  batch=PBS)
task1 = Task(
    command="lmp_serial -i input.lammps",
    task_work_path="bct-1",
    forward_files=["conf.lmp", "input.lammps"],
    backward_files=["log.lammps"],
)
task2 = Task(
    command="lmp_serial -i input.lammps",
    task_work_path="bct-2",
    forward_files=["conf.lmp", "input.lammps"],
    backward_files=["log.lammps"],
)
task3 = Task(
    command="lmp_serial -i input.lammps",
    task_work_path="bct-3",
    forward_files=["conf.lmp", "input.lammps"],
    backward_files=["log.lammps"],
)
task4 = Task(
    command="lmp_serial -i input.lammps",
    task_work_path="bct-4",
    forward_files=["conf.lmp", "input.lammps"],
    backward_files=["log.lammps"],
)
submission.register_task_list(
    [
        task1,
        task2,
        task3,
        task4,
    ]
)
submission.generate_jobs()
submission.bind_batch(batch=batch)
# for job in submission.belonging_jobs:
#     job.job_to_json()
# print('111', submission)
# submission2 = Submission.recover_jobs_from_json('./jr.json')
# print('222', submission2)
# print(submission==submission2)
submission.run_submission()

# submission1.dump_jobs_fo_json()
# submission2 = Submission.submission_from_json('jsons/submission.json')
# print(677, submission==submission2)
# print(submission1.belonging_jobs)
# print(local_context)



# /usr/bin/python
# -*- encoding=utf-8 -*-

import os
import sys

from dpdispatcher.utils import run_cmd_with_all_output


class HDFS(object):
    """Fundamental class for HDFS basic manipulation"""

    @staticmethod
    def exists(uri):
        """Check existence of hdfs uri
        Returns: True on exists
        Raises: RuntimeError
        """
        cmd = "hadoop fs -test -e {uri}".format(uri=uri)
        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return True
            elif ret == 1:
                return False
            else:
                raise RuntimeError(
                    "Cannot check existence of hdfs uri[{}] "
                    "with cmd[{}]; ret[{}] stdout[{}] stderr[{}]".format(
                        uri, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot check existence of hdfs uri[{}] "
                "with cmd[{}]".format(uri, cmd)
            ) from e

    @staticmethod
    def remove(uri):
        """Check existence of hdfs uri
        Returns: True on exists
        Raises: RuntimeError
        """

        cmd = "hadoop fs -rm -r {uri}".format(uri=uri)
        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return True
            else:
                raise RuntimeError(
                    "Cannot remove hdfs uri[{}] "
                    "with cmd[{}]; ret[{}] output[{}] stderr[{}]".format(
                        uri, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot remove hdfs uri[{}] " "with cmd[{}]".format(uri, cmd)
            ) from e

    @staticmethod
    def mkdir(uri):
        """Make new hdfs directory
        Returns: True on success
        Raises: RuntimeError
        """
        cmd = "hadoop fs -mkdir -p {uri}".format(uri=uri)
        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return True
            else:
                raise RuntimeError(
                    "Cannot mkdir of hdfs uri[{}] "
                    "with cmd[{}]; ret[{}] output[{}] stderr[{}]".format(
                        uri, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot mkdir of hdfs uri[{}] " "with cmd[{}]".format(uri, cmd)
            ) from e

    @staticmethod
    def copy_from_local(local_path, to_uri):
        """
        Returns: True on success
        Raises: on unexpected error
        """
        # Make sure local_path is accessible
        if not os.path.exists(local_path) or not os.access(local_path, os.R_OK):
            raise RuntimeError(
                "try to access local_path[{}] " "but failed".format(local_path)
            )
        cmd = "hadoop fs -copyFromLocal -f {local} {remote}".format(
            local=local_path, remote=to_uri
        )
        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return True, out
            else:
                raise RuntimeError(
                    "Cannot copy local[{}] to remote[{}] with cmd[{}]; "
                    "ret[{}] output[{}] stderr[{}]".format(
                        local_path, to_uri, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot copy local[{}] to remote[{}] with cmd[{}]".format(
                    local_path, to_uri, cmd
                )
            ) from e

    @staticmethod
    def copy_to_local(from_uri, local_path):
        remote = ""
        if isinstance(from_uri, str):
            remote = from_uri
        elif isinstance(from_uri, list) or isinstance(from_uri, tuple):
            remote = " ".join(from_uri)
        cmd = "hadoop fs -copyToLocal {remote} {local}".format(
            remote=remote, local=local_path
        )

        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return True
            else:
                raise RuntimeError(
                    "Cannot copy remote[{}] to local[{}] with cmd[{}]; "
                    "ret[{}] output[{}] stderr[{}]".format(
                        from_uri, local_path, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot copy remote[{}] to local[{}] with cmd[{}]".format(
                    from_uri, local_path, cmd
                )
            ) from e

    @staticmethod
    def read_hdfs_file(uri):
        cmd = "hadoop fs -text {uri}".format(uri=uri)
        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return out
            else:
                raise RuntimeError(
                    "Cannot read text from uri[{}]"
                    "cmd [{}] ret[{}] output[{}] stderr[{}]".format(
                        uri, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot read text from uri[{}]" "cmd [{}]".format(uri, cmd)
            ) from e

    @staticmethod
    def move(from_uri, to_uri):
        cmd = "hadoop fs -mv {furi} {turi}".format(furi=from_uri, turi=to_uri)
        try:
            ret, out, err = run_cmd_with_all_output(cmd)
            if ret == 0:
                return True
            else:
                raise RuntimeError(
                    "Cannot move from_uri[{}] to "
                    "to_uri[{}] with cmd[{}]; "
                    "ret[{}] output[{}] stderr[{}]".format(
                        from_uri, to_uri, cmd, ret, out, err
                    )
                )
        except Exception as e:
            raise RuntimeError(
                "Cannot move from_uri[{}] to "
                "to_uri[{}] with cmd[{}]".format(from_uri, to_uri, cmd)
            ) from e
